class GeneralFunctionsQueryBuilder {
  constructor(elements) {
    this.elements = elements;
  }

  toggleClass(elements, className, action){
    elements.forEach(element => {
      document.querySelector('[data-toggle-area="' + element + '"]').classList[action](className);
    });
  }

  updateChipList() {
    this.elements.queryChipElements = this.elements.queryInputField.querySelectorAll('.chip');
  }

  queryChipFactory(dataType, prettyQueryText, queryText, index = null) {
    queryText = Utils.escape(queryText);
    prettyQueryText = Utils.escape(prettyQueryText);
    let queryChipElement = Utils.HTMLToElement(
      `
        <span class="chip query-component" data-type="${dataType}" data-query="${queryText}" draggable="true">
          ${prettyQueryText}
          <i class="material-icons close">close</i>
        </span>
      `
    );

    queryChipElement.addEventListener('click', () => this.selectChipElement(queryChipElement));
    queryChipElement.querySelector('i').addEventListener('click', () => this.deleteChipElement(queryChipElement));
    
    queryChipElement.addEventListener('dragstart', this.handleDragStart.bind(this, queryChipElement));
    queryChipElement.addEventListener('dragend', this.handleDragEnd);
    if (index !== null) {
      this.elements.queryInputField.insertBefore(queryChipElement, this.elements.queryChipElements[index]);
    } else {
      this.elements.queryInputField.appendChild(queryChipElement);
    }
    this.updateChipList();
    this.queryPreviewBuilder();
  }

  handleDragStart(queryChipElement, event) {
    let queryChips = this.elements.queryInputField.querySelectorAll('.query-component');
    setTimeout(() => {
      let targetChipElement = Utils.HTMLToElement('<span class="chip drop-target">Drop here</span>');
      for (let element of queryChips) {
        if (element === queryChipElement.nextSibling) {continue;}
        let targetChipClone = targetChipElement.cloneNode(true);
        if (element === queryChipElement && queryChips[queryChips.length - 1] !== element) {
          queryChips[queryChips.length - 1].insertAdjacentElement('afterend', targetChipClone);
        } else {
          element.insertAdjacentElement('beforebegin', targetChipClone);
        }
        this.addDragDropListeners(targetChipClone, queryChipElement);
      }
    }, 0);
  }

  handleDragEnd(event) {
    document.querySelectorAll('.drop-target').forEach(target => target.remove());
  }

  addDragDropListeners(targetChipClone, queryChipElement) {
    targetChipClone.addEventListener('dragover', (event) => {
      event.preventDefault();
    });
    targetChipClone.addEventListener('dragenter', (event) => {
      event.preventDefault();
      event.target.style.borderStyle = 'solid dotted';
    });
    targetChipClone.addEventListener('dragleave', (event) => {
      event.preventDefault();
      event.target.style.borderStyle = 'hidden';
    });
    targetChipClone.addEventListener('drop', (event) => {
      let dropzone = event.target;
      dropzone.parentElement.replaceChild(queryChipElement, dropzone);
      this.updateChipList();
      this.queryPreviewBuilder();
    });
  }

  queryPreviewBuilder() {
    let queryPreview = document.querySelector('#corpus-analysis-concordance-query-preview');
    let queryInputFieldContent = [];
    this.elements.queryChipElements.forEach(element => {
      let queryElement = element.dataset.query;
      if (queryElement !== undefined) {
        queryElement = Utils.escape(queryElement);
      }
      queryInputFieldContent.push(queryElement);
    });
  
    let queryString = queryInputFieldContent.join(' ');
    if (queryString.includes(' +')) {
      queryString = queryString.replace(/ \+/g, '+');
    }
    if (queryString.includes(' *')) {
      queryString = queryString.replace(/ \*/g, '*');
    }
    if (queryString.includes(' ?')) {
      queryString = queryString.replace(/ \?/g, '?');
    }
    if (queryString.includes(' {')) {
      queryString = queryString.replace(/ \{/g, '{');
    }
    queryString += ';';
  
    queryPreview.innerHTML = queryString;
    queryPreview.parentNode.classList.toggle('hide', queryString === ';');
  }

  deleteChipElement(attr) {
    if (attr.dataset.type === "start-sentence") {
      this.elements.sentenceElement.innerHTML = 'Sentence';
    } else if (attr.dataset.type === "start-entity" || attr.dataset.type === "start-empty-entity") {
      this.elements.entityElement.innerHTML = 'Entity';
    }
    this.elements.queryInputField.removeChild(attr);
    this.updateChipList();
    this.queryPreviewBuilder();
  }

  selectChipElement(attr) {
    document.querySelectorAll('.chip.teal').forEach(element => {
        if (element !== attr) {
          element.classList.remove('teal', 'lighten-2');
          this.toggleClass(['token-incidence-modifiers'], 'disabled', 'add');
        }
    });

    this.toggleClass(['token-incidence-modifiers'], 'disabled', 'toggle');
    attr.classList.toggle('teal');
    attr.classList.toggle('lighten-5');
  }

  tokenIncidenceModifierHandler(incidenceModifier, incidenceModifierPretty) {
    let selectedChip = this.elements.queryInputField.querySelector('.chip.teal');
    let selectedChipIndex = Array.from(this.elements.queryInputField.children).indexOf(selectedChip);
    this.queryChipFactory('token-incidence-modifier', incidenceModifierPretty, incidenceModifier, selectedChipIndex+1);
    this.selectChipElement(selectedChip);
  }

  tokenNMSubmitHandler(modalId) {
    let modal = document.querySelector(`#${modalId}`);
    let input_n = modal.querySelector('.n-m-input[data-value-type="n"]').value;
    let input_m = modal.querySelector('.n-m-input[data-value-type="m"]') || undefined;
    input_m = input_m !== undefined ? input_m.value : '';
    let input = `{${input_n},${input_m}}`;
    let pretty_input = `between ${input_n} and ${input_m} (${input})`;
    if (input_m === '') {
     pretty_input = `exactly ${input_n} (${input})`;
    }

    let instance = M.Modal.getInstance(modal);
    instance.close();

    this.tokenIncidenceModifierHandler(input, pretty_input);

  }
}

